;                        Multiface 1 RAM Test v. 1.0
;                        ===========================
;File: RAM_Test_1.0.asm
;Version: 1.0 - Aug/31/2011
;Author: Flavio M Matsumoto (reikainosuke-tk@yahoo.com.br)
;Description:
; This program applies 4 type of tests on Multiface 1 built-in 8 kilobytes of
;RAM (8192-16383/#2000-#3FFF). 
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
        ORG 30000
        DI                      ; Disable interrupts and switch to M1 ROM.
        LD DE,CharMap           ; Copy ROM character bitmap table to RAM.
        LD HL,15616
        LD BC,768
        LDIR
        IN A,(159)
        CALL Delay              ; Wait.
        LD BC,8192              ; 8192 is both M1 RAM start address and its
        PUSH BC                 ;lenght.
        PUSH BC
        PUSH BC                 ; IX points to start of M1 RAM.
        POP IX
; First, fill all M1 RAM with value 1.
TestA:
        LD (IX+0),1             ; Fill all M1 RAM with 1.
        INC IX                  ; Increment RAM pointer.
        DEC BC                  ; Decrement counter.
        LD A,B
        OR C
        JR NZ,TestA             ; Repeat until counter is 0.
; All bytes of RAM are decremented to 0. RAM is scanned backwards.
        LD HL,18432             ; Print 'Test 1: decrement to 0'
        LD DE,Msg003
        LD BC,Msg004-Msg003
        CALL PrStrS
        LD HL,18432+32          ; Print 'Testing address: #'.
        LD DE,Msg001
        LD BC,Msg002-Msg001
        CALL PrStrS
        CALL Delay              ; Wait.
        POP BC                  ; BC=8192, lenght of M1 RAM.
TestA1:
        PUSH BC                 ; Save counter.
        DEC IX                  ; Decrement RAM pointer.
        PUSH IX                 ; Address is copied to BC and
        POP BC
        PUSH HL
        CALL PrHexBC            ;printed. 
        POP HL
        DEC (IX+0)              ; Byte in RAM must be 0 now.
        CALL NZ,Error           ; If not then print error message.
        POP BC                  ; Restore counter.
        DEC BC                  ; Decrement counter.
        LD A,B
        OR C
        JR NZ,TestA1            ; Repeat until counter is 0.
; All bytes of RAM are further decremented to -1 (255).
        CALL Delay              ; Wait.
        PUSH HL
        LD HL,18432             ; Print 'Test 2: decrement to -1'
        LD DE,Msg004
        LD BC,Msg005-Msg004
        CALL PrStrS
        POP HL
        POP BC                  ; BC=8192, lenght of M1 RAM.
        PUSH BC
        PUSH HL                 ; Save print position.

TestA2:
        PUSH BC                 ; Save counter.
        INC IX                  ; Increment RAM pointer.
        PUSH IX                 ; Address is copied to BC and
        POP BC
        PUSH HL
        CALL PrHexBC            ;printed.
        POP HL
        DEC (IX+0)              ; Byte in RAM must be negative now.
        CALL P,Error            ; If not then print error message.
        POP BC                  ; Restore counter.
        DEC BC                  ; Decrement counter.
        LD A,B
        OR C
        JR NZ,TestA2
; Write a bit pattern to each byte, and its complement to the adjacent bytes.
        CALL Delay              ; Wait.
        LD HL,18432
        LD DE,Msg005            ;'Test 3: write %01010101'
        LD BC,Msg006-Msg005
        CALL PrStrS
        POP HL
        POP BC                  ; BC=8192, lenght of M1 RAM.
        PUSH BC
        PUSH HL                 ; Save print position.
TestA3:
        PUSH BC                 ; Save counter.
        DEC IX                  ; Decrement RAM pointer.
        PUSH IX                 ; Address is copied to BC and
        POP BC
        PUSH HL
        CALL PrHexBC            ;printed.
        POP HL
        LD A,%01010101          ; Write bit pattern to the byte.
        LD (IX+0),A
        CPL                     ; Write complement to adjacent bytes. 
        LD (IX-1),A
        LD (IX+1),A
        CPL
        CP (IX+0)               ; Test byte integrity.
        CALL NZ,Error           ; If failed then print error message.
        POP BC                  ; Restore counter.
        DEC BC                  ; Decrement counter.
        LD A,B
        OR C
        JR NZ,TestA3
; Complement RAM value 
        CALL Delay              ; Wait.
        LD HL,18432
        LD DE,Msg006            ; Print 'Test 4: complement RAM'
        LD BC,Msg007-Msg006
        CALL PrStrS
        POP HL
        POP BC
TestA4:
        PUSH BC                 ; Save counter.
        INC IX                  ; Increment RAM pointer.
        PUSH IX                 ; Address is copied to BC and
        POP BC
        PUSH HL
        CALL PrHexBC            ;printed.
        POP HL
        LD A,(IX+0)             ; Complement byte.
        CPL
        LD (IX+0),A
        LD (IX-1),0             ; Zero previous byte.
        CP (IX+0)               ; Test byte integrity.
        CALL NZ,Error           ; If failed then print error message.
        POP BC                  ; Restore counter.
        DEC BC                  ; Decrement counter.
        LD A,B
        OR C
        JR NZ,TestA4
;
        CALL Delay              ; Wait.
        IN A,(31)               ; Switch back Spectrum ROM.
        EI                      ; Interrupts are enabled.
        RET                     ; End of the program.
;
Error:
        PUSH HL                 ; Print position pointer and
        PUSH BC                 ;counter are saved.
        LD HL,20480+32          ; Print at 16,0;
        PUSH HL                 ; Save print position.
        LD DE,Msg002            ; Print "Error! Press any key!"
        LD BC,Msg003-Msg002
        CALL PrStrS
Error1:
        CALL RdKbd              ; Repeat
        JR Z,Error1             ;until a key is pressed.
        POP HL                  ; Restore print position (at 16,0;) and
        LD B,32                 ;print 32 blank spaces to erase line.
Error2:
        LD A," "
        PUSH BC
        CALL PrAChrS
        POP BC
        DJNZ Error2
        POP BC                  ; Restore print position pointer and counter.
        POP HL
        RET                     ; End of sub-routine.
;
RdKbd:
        XOR A                   ; Read all key rows. 
        IN A,(254)
        CPL 
        AND %00011111
        RET
;
Delay:
        LD B,0
Delay1:
        DJNZ Delay1
        RET
;
;------------------------------------------------------------------------------
; Print ASCII Char (PrtAChr) - print a ASCII character
; Input:    A=ASCII character code (32 to 127)
;           HL=screen address (16384 to 22527)
; Output:   HL=address of next print position
;           Flags:  Z=1 if last print position of screen area was surpassed
;                   C=1 if last print position of screen area was surpassed
; Used registers: AF, BC, DE, HL
; Note: modified to prevent use of ROM character bitmap model. 
;------------------------------------------------------------------------------
;
PrtAChr:
        LD BC,CharMap-256   ; BC points to character bitmap model table.
PrtAChr1:               ; Another entry-point (BC has to be assigned before).
        PUSH HL         ; Save screen address
        EX DE,HL        ;and transfer to DE.
        CP 32           ; If A<32 then jump forward to print "?".
        JR C,PrtAChr2
        CP 127          ; If A<=127 then jump forward, else print "?".
        JR C,PrtAChr3
PrtAChr2:
        LD A,"?"        ; Imprimir "?".
PrtAChr3:
        LD H,0          ; HL=code of character (32-127). 
        LD L,A
        ADD HL,HL       ; HL=HL*8, each character bitmap model in the table has
        ADD HL,HL       ;8 bytes.
        ADD HL,HL
        ADD HL,BC       ; Add to BC to get character bitmap model address.
        LD B,8          ; Assign counter for 8 bytes.
PrtAChr4:
        LD A,(HL)       ; Copy each byte to screen.
        LD (DE),A       ;
        INC HL          ; HL points to next bitmap model byte.
        INC D           ; DE points to next screen pixel line.
        DJNZ PrtAChr4   ; Repeat until last byte.
        POP HL          ; Restore screen address and increment to get next 
        INC L           ;address of next column, but if the last position of the
                        ;screen area was surpassed, HL points to the first print
                        ;position and Z/C flags are set.
        RET             ; Return.
;
;------------------------------------------------------------------------------
; Print ASCII Char Screen (PrAChrS) - print a ASCII character (whole screen)
; Input:    A=ASCII character code (32 to 127)
;           HL=screen address (16384 to 22527)
; Output:   HL=address of next print position
;           Flags:  Z=1 if last print position of screen area was surpassed
; Used registers: AF, BC, DE, HL
; Requires: PrtAChr
;------------------------------------------------------------------------------
;
PrAChrS:
        CALL PrtAChr        ; Print character.
        RET NZ              ; If it wasn't last position then return.
        LD A,H
        ADD A,%00001000     ; Go to next screen area.
        CP    %01011000     ; If screen last screen position wasn't surpassed
        JR NZ,PrAChrS1      ;then jump forward,
        LD A,%01000000      ;else go back to start of the screen.
PrAChrS1:
        LD H,A
        RET                 ; Return.
;
;------------------------------------------------------------------------------
; Print String Screen (PrStrS) - print a string (whole screen)
; Input:    DE=string address 
;           BC=string lenght
;           HL=screen address (16384 to 22527)
; Output:   HL=next print address
;           DE=address after string
;           BC=0
; Requires: PrAChrS
;------------------------------------------------------------------------------
;
PrStrS:
        LD A,(DE)       ; Put string character into A.
        PUSH DE         ; Save registers.
        PUSH BC
        CALL PrtAChr    ; Print character.
        POP BC          ; Restore registers.
        POP DE
        INC DE          ; Point to next character of string.
        DEC BC          ; Decrement string length count.
        LD A,B          ; Repeat until last byte of string.
        OR C
        JR NZ,PrStrS
        RET             ; Return.
;
;------------------------------------------------------------------------------
; Print Hexadecimal BC (PrHexBC) - print hexadecimal number in BC (whole screen)
; Input:    BC=number to be printed
;           HL=screen address (16384 to 22527)
; Output:   HL=address of next print position
;           Flags:  Z=1 if last print position of screen area was surpassed
; Used registers: AF, BC, DE, HL
; Requires: PrAChrS
;            PrHexA
; Adapted from: 'http://baze.au.com/misc/z80bits.html#5.1'. 
;------------------------------------------------------------------------------
;
PrHexBC:
        LD A,B
        PUSH BC
        CALL PrHexA
        POP BC
        LD A,C
;
;------------------------------------------------------------------------------
; Print Hexadecimal A (PrHexA) - print hexadecimal number in A (whole screen)
; Input:    A=number to be printed
;           HL=screen address (16384 to 22527)
; Output:   HL=address of next print position
;           Flags:  Z=1 if last print position of screen area was surpassed
; Used registers: AF, BC, DE, HL
; Requires: PrAChrS
;------------------------------------------------------------------------------
;
PrHexA:
        PUSH AF         ; Save A.
        RLCA            ; Tranfer most significant nibble (MSN) into
        RLCA            ;least signficant nibble (LSN).
        RLCA
        RLCA
        CALL PrHexA1    ;Print MSN.
        POP AF          ;Recover A. 
PrHexA1:
        AND %00001111   ; MSN bits are reset.
        OR  %00110000   ; Convert digit to ASCII.
        CP '9'+1        ; If A<='9'
        JR C,PrHexA2    ;then jump the next instruction,
        ADD A,7         ;else put A into the range 'A'-'F'. 
PrHexA2:
        CALL PrAChrS    ; Print A. 
        RET             ; Return.
;
; Messages
Msg001:
        DEFM 'Testing address: #'
Msg002:
        DEFM 'Error! Press any key!'
Msg003:
        DEFM 'Test 1: decrement to 0 '
Msg004:
        DEFM 'Test 2: decrement to -1'
Msg005:
        DEFM 'Test 3: write %01010101'
Msg006:
        DEFM 'Test 4: complement RAM '
Msg007:
CharMap:
        DEFB 0
